/**
 * @license
 * Copyright Akveo. All Rights Reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */
import React from 'react';
import { findNodeHandle, UIManager, Dimensions, } from 'react-native';
import { Frame } from './type';
/**
 * Measures child element size and it's screen position asynchronously.
 * Returns measure result in `onResult` callback.
 *
 * Usage:
 *
 * const onMeasure = (element: ElementToMeasure): void => {
 *   const { x, y, width, height } = element.props.frame;
 *   ...
 * };
 *
 * <MeasureElement onResult={this.onMeasure}>
 *   <ElementToMeasure tag='@measure/measure-me!'/>
 * </MeasureElement>
 */
export const MeasureElement = (props) => {
    const ref = React.createRef();
    const { children, onResult } = props;
    const bindToWindow = (frame, window) => {
        if (frame.origin.x < window.width) {
            return frame;
        }
        const boundFrame = new Frame(frame.origin.x - window.width, frame.origin.y, frame.size.width, frame.size.height);
        return bindToWindow(boundFrame, window);
    };
    const measureSelf = () => {
        const node = findNodeHandle(ref.current);
        UIManager.measureInWindow(node, (x, y, w, h) => {
            const window = Dimensions.get('window');
            const frame = bindToWindow(new Frame(x, y, w, h), window);
            const measuredElement = React.cloneElement(children, { frame });
            onResult(measuredElement);
        });
    };
    return React.cloneElement(children, {
        ref,
        onLayout: measureSelf,
    });
};
/**
 * Measures passed child elements size and it's screen position asynchronously.
 * Returns measure result in `onResult` callback.
 *
 * Does the same as `MeasureElement` but calls `onResult` after all children are measured.
 *
 * Usage:
 *
 * const onMeasure = (result: MeasureResult): void => {
 *   const { [0]: firstElementFrame, [1]: secondElementFrame } = result;
 *   const { x, y, width, height } = firstElementFrame;
 *   ...
 * };
 *
 * <MeasureNode onResult={this.onMeasure}>
 *   <ElementToMeasure tag={0}/>
 *   <ElementToMeasure tag={1}/>
 * </MeasureNode>
 */
export const MeasureNode = (props) => {
    const result = {};
    const { children, onResult } = props;
    const onChildMeasure = (element) => {
        const { tag, frame } = element.props;
        result[tag] = frame;
        if (Object.keys(result).length === React.Children.count(children)) {
            onResult(result);
        }
    };
    const renderMeasuringElement = (element, index) => {
        return (<MeasureElement key={index} onResult={onChildMeasure}>
        {element}
      </MeasureElement>);
    };
    const renderChildren = () => {
        return React.Children.map(children, renderMeasuringElement);
    };
    return (<React.Fragment>
      {renderChildren()}
    </React.Fragment>);
};
//# sourceMappingURL=measure.component.js.map